import os
import json
import logging
from typing import List
from models.models import HAResult, HADocument
from alibabacloud_ha3engine_vector import models, client
from .basestore import BaseStore

class AliyunOpenSearch(BaseStore):
    ENDPOINT = os.environ.get('OPENSEARCH_ENDPOINT')
    INSTANCE_ID = os.environ.get('OPENSEARCH_INSTANCE_ID')
    TABLE_NAME = os.environ.get('OPENSEARCH_TABLE_NAME')
    USERNAME = os.environ.get('OPENSEARCH_USER_NAME')
    PASSWORD = os.environ.get('OPENSEARCH_PASSWORD')

    def __init__(self) -> None:
        endpoint = self.ENDPOINT
        if endpoint.startswith("http://"):
            endpoint = endpoint[7:]

        self.config = models.Config(
            endpoint=endpoint,
            instance_id=self.INSTANCE_ID,
            access_user_name=self.USERNAME,
            access_pass_word=self.PASSWORD
        )

    async def insert(self, ha_docs: List[HADocument]) -> None:
        engine_client = client.Client(self.config)
        options_headers = {}

        pk_field = 'pk'
        documents = [ha_doc.to_os_doc() for ha_doc in ha_docs]
        request = models.PushDocumentsRequest(options_headers, documents)
        data_source = self.INSTANCE_ID + "_" + self.TABLE_NAME
        await engine_client.push_documents_async(data_source, pk_field, request)

    async def query(self, embedding: List[float], topk: int) -> List[HAResult]:
        engine_client = client.Client(self.config)
        result = []
        try:
            request = models.QueryRequest(table_name=self.TABLE_NAME,
                                          vector=embedding,
                                          include_vector=False,
                                          top_k=topk,
                                          output_fields=['content', 'source_id'])
            response = await engine_client.query_async(request)
            obj = json.loads(response.body)
            if 'errorCode' in obj:
                raise RuntimeError(f"query opensearch failed, {obj['errorMsg']}")
            result = [HAResult(pk=item['id'], content=item['fields']['content'], source_id=item['fields']['source_id']) for item in obj['result']]
            return result
        except Exception as e:
            logging.exception(f"search opensearch failed: {e}")
        finally:
            return result